;;; fpkg --- a simple package manager for FG42                     -*- lexical-binding: t; -*-
;;
;; Copyright (C) 2010-2024  Sameer Rahmani <lxsameer@gnu.org>
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; Keywords: lisp fg42 IDE package manager
;; Version: 1.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;
;; Simple package manager for FG42
;;
;;; Code:

(require 'map)

(defvar package-names ())

(eval-when-compile
  (defvar fg42-use-nix)) ;; defined in fg42-config.el

(defun inject-straight (args)
  "Inject `:straight t' to ARGS it the key was missing."
  (if fg42-use-nix
      ;; We don't use straight with nix
      (map-delete args :straight)
    (if (member :straight args)
        args
      (append args '(:straight t)))))


(defun inject-params (args)
  "Inject required `use-package' params to ARGS if the key is missing."
  (if fg42-use-nix
      (append '(:ensure nil) args)
    ;; (if (member :defer args)
    ;;     args
    ;;   (append args '(:defer t)))
    args))


(defmacro fpkg/use (pkg &rest details)
  "Install the given package DETAILS PKG via `use-package' and straight."
  (declare (indent defun))
  (if (and (listp details) (< 0 (length details)))
    (let ((p (inject-straight (inject-params details))))
      `(progn
         (use-package ,pkg ,@p)))
    `(progn
       (use-package ,pkg :defer t :ensure nil))))


(defmacro fpkg/require (pkg)
  "Work like require but make sure that PKG is installed first."

  (let ((pkg-name (intern (symbol-name `,(cadr pkg)))))
    (if fg42-use-nix
        `(require ,pkg)
      `(fpkg/use ,pkg-name
         :init
         (require ,pkg)))))

(provide 'fpkg)
;;; fpkg.el ends here
